@@ -1,6 +1,6 @@
 # !/usr/bin/env python
 # -*- coding: utf-8 -*-
-# @Time    : 2023/01/25
+# @Time    : 2023/01/27
 # @Author  : longchengguxiao
 # @File    : nonebot_plugin_pvz
 # @Version : 3.8.9 Python
@@ -14,13 +14,15 @@ from nonebot.params import CommandArg, ArgStr
 from nonebot import on_command
 import nonebot
 from nonebot import require
+from nonebot.permission import SUPERUSER
 from nonebot_plugin_apscheduler import scheduler
 from pathlib import Path
 from collections import Counter
 import json
 import asyncio
 import os
-from nonebot_plugin_pvz.config import Config
+import shutil
+from .config import Config
 
 # 启动定时器
 
@@ -32,8 +34,19 @@ require("nonebot_plugin_apscheduler")
 
 global_config = nonebot.get_driver().config
 pvz_config = Config.parse_obj(global_config.dict())
-pvz_basic_path = pvz_config.pvz_basic_path / \
-    os.path.dirname(os.path.abspath(__file__))
+pvz_basic_path = pvz_config.pvz_basic_path
+
+if pvz_basic_path != Path() and not os.path.exists(os.path.join(pvz_basic_path, 'font', 'msyh.ttf')):
+    shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'font'),
+                    os.path.join(pvz_basic_path, 'font'), dirs_exist_ok=True)
+    shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'images'),
+                    os.path.join(pvz_basic_path, 'images'), dirs_exist_ok=True)
+    shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'user_data'),
+                    os.path.join(pvz_basic_path, 'user_data'), dirs_exist_ok=True)
+
+if pvz_basic_path == Path():
+    pvz_basic_path = pvz_basic_path / os.path.dirname(os.path.abspath(__file__))
+
 
 bag_path = pvz_basic_path / "user_data" / "bag.txt"
 lawn_path = pvz_basic_path / "user_data" / "lawn.txt"
@@ -44,6 +57,7 @@ PVZ_IMAGE_PATH = pvz_basic_path / 'images' / 'pvz' / 'base'
 PVZ_OUTPUT_PATH = pvz_basic_path / 'images' / 'pvz' / 'output'
 PVZ_ORI_PATH = pvz_basic_path / 'images' / 'pvz' / 'ori'
 
+
 STATE_OK = True
 STATE_ERROR = False
 
@@ -95,6 +109,7 @@ class Plants():  # 植物类
         self.penetrable = 0
         self.only_hurt_by_boss = 0
         self.only_hurt_metal = 0
+        self.unjumpable = 0
 
         # 豌豆射手
         if plantname == "pea_shooter":
@@ -326,6 +341,7 @@ class Plants():  # 植物类
         self.penetrable = 0
         self.only_hurt_by_boss = 0
         self.only_hurt_metal = 0
+        self.unjumpable = 1
 
     def cabbage_pult(self):
         """
@@ -778,9 +794,13 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
         while zb.hp > all_damage:
             # 计算当前僵尸距每个植物的距离
             distance = [dist - pos for pos in plant_pos if pos < dist]
+            if not distance:
+                iswin = 1
+                break
             # 以0.5为僵尸的攻击距离
             if min(distance) > 0.5 or (lawn_plants[distance.index(
                     min(distance))].only_hurt_by_boss == 1 and zb.is_gargantuar == 0):
+                # 此时僵尸攻击不到植物，仅植物攻击僵尸
                 # 距离递减
                 dist -= zb.v * 0.5
                 # 计算植物伤害
@@ -795,6 +815,7 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                 # 记录总伤害，便于打印日志
                 all_damage += damage
             else:
+                # 此时植物和僵尸可以互相攻击
                 # 当前目标植物
                 tar = distance.index(min(distance))
                 # 变成战斗状态
@@ -826,15 +847,9 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                     fight_time += 0.5
                 # 判断僵尸是否可以跳过植物
                 if zb.jump:
-                    # 距离递减
-                    dist -= 1
-                    # 生成图片
-                    lawn_pic([all_plants[x] for x in plants if x != "0"], cnt)
-                    zombie_pic(all_zombie[zombie], dist, cnt)
-                    # 如果是跳跳僵尸则不会失去下一次跳跃的功能
-                    if zb.v == 0.4:
+                    if lawn_plants[tar].unjumpable == 1:
                         zb.jump = 0
-                        zb.v = 0.2
+                        zb.v = zb.v/2
                         log.append({
                             "type": "node",
                             "data": {
@@ -844,44 +859,69 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                                     {
                                         "type": "text",
                                         "data": {
-                                            "text": f"时间{time},{zombie}被植物阻挡后起跳，撑杆掉落，速度减半"
+                                            "text": f"时间{time},{zombie}被高坚果阻挡后无法起跳，撑杆掉落，速度减半"
                                         }
                                     },
-                                    {
-                                        "type": "image",
-                                        "data": {
-                                            "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
-                                        }
-                                    }
                                 ]
                             }
-                        }
-                        )
+                        })
                     else:
-                        # 如果是跳跳僵尸，则可以继续跳过，同时失去攻击机会
-                        log.append({
-                            "type": "node",
-                            "data": {
-                                "name": "疯狂戴夫",
-                                "uin": "1298919732",
-                                "content": [
-                                    {
-                                        "type": "text",
-                                        "data": {
-                                            "text": f"时间{time},{zombie}被植物阻挡后起跳,同时企图继续跳过其他植物"
+                        # 距离递减
+                        dist -= 1
+                        # 生成图片
+                        lawn_pic([all_plants[x] for x in plants if x != "0"], cnt)
+                        zombie_pic(all_zombie[zombie], dist, cnt)
+                        # 如果是跳跳僵尸则不会失去下一次跳跃的功能
+                        if zb.v == 0.4:
+                            zb.jump = 0
+                            zb.v = 0.2
+                            log.append({
+                                "type": "node",
+                                "data": {
+                                    "name": "疯狂戴夫",
+                                    "uin": "1298919732",
+                                    "content": [
+                                        {
+                                            "type": "text",
+                                            "data": {
+                                                "text": f"时间{time},{zombie}被植物阻挡后起跳，撑杆掉落，速度减半"
+                                            }
+                                        },
+                                        {
+                                            "type": "image",
+                                            "data": {
+                                                "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
+                                            }
                                         }
-                                    },
-                                    {
-                                        "type": "image",
-                                        "data": {
-                                            "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
-                                        }
-                                    }
-                                ]
+                                    ]
+                                }
                             }
-                        }
-                        )
-                        continue
+                            )
+                        else:
+                            # 如果是跳跳僵尸，则可以继续跳过，同时失去攻击机会
+                            log.append({
+                                "type": "node",
+                                "data": {
+                                    "name": "疯狂戴夫",
+                                    "uin": "1298919732",
+                                    "content": [
+                                        {
+                                            "type": "text",
+                                            "data": {
+                                                "text": f"时间{time},{zombie}被植物阻挡后起跳,同时企图继续跳过其他植物"
+                                            }
+                                        },
+                                        {
+                                            "type": "image",
+                                            "data": {
+                                                "file": f"file:///{PVZ_OUTPUT_PATH}/output_new{cnt}.png"
+                                            }
+                                        }
+                                    ]
+                                }
+                            }
+                            )
+                        cnt += 1
                     # 判断当前目标
                     if tar > 0:
                         lawn_plants[tar - 1].hp -= zb.damage
@@ -928,7 +968,7 @@ def one_by_one(team: List[str], plants: List[str]) -> (List, int):
                             "data": {
                                 "name": "疯狂戴夫",
                                 "uin": "1298919732",
-                                "content": f"时间{time},{zombie}对{plants[tar]}造成{zb.damage}点伤害。两次攻击间植物合计对{zombie}造成{all_damage}点伤害,{zombie}剩余血量为{zb.hp - all_damage}"
+                                "content": f"时间{time},{zombie}对{plants[tar]}造成{zb.damage if zb.jump == 0 else 0}点伤害。两次攻击间植物合计对{zombie}造成{all_damage}点伤害,{zombie}剩余血量为{zb.hp - all_damage}"
                             }
                         }
                         )
@@ -1058,9 +1098,34 @@ put_on_lawn = on_command("放置", block=True, priority=5)
 play_with_computer_zombie = on_command("植物人机训练", block=True, priority=5)
 play_with_computer_plant = on_command(
     "僵尸人机训练", block=True, priority=5, aliases={"我是僵尸"})
-_help = on_command("植物大战僵尸帮助", aliases={"pvz帮助"}, priority=5, block=True)
+_help = on_command(
+    "植物大战僵尸帮助",
+    aliases={
+        "pvz帮助",
+        "pvz使用说明",
+        "pvz使用方法"},
+    priority=5,
+    block=True)
 fight = on_command("入侵", priority=5, block=True)
 
+download_data = on_command(
+    "pvz下载数据",
+    priority=5,
+    block=True,
+    permission=SUPERUSER,
+    aliases={
+        "pvz保存数据",
+        "pvz数据保存",
+        "pvz数据下载"})
+upload_data = on_command(
+    "pvz载入数据",
+    priority=5,
+    block=True,
+    permission=SUPERUSER,
+    aliases={
+        "pvz上传数据",
+        "pvz数据载入",
+        "pvz数据上传"})
 
 # pvz签到-----------------------------------------------------------------------------
 
@@ -1407,10 +1472,11 @@ async def _(state: T_State, cate: str = ArgStr("cate")):
         pos = int(pos)
         if plant in list(all_plants.keys()):
             now_pos = state["lawn_puts"].split(',')
-            my_plants: List = state["plants"]
+            my_plants: List = (state["plants"]).split(",")
             if plant in my_plants:
                 for p in now_pos:
-                    my_plants.remove(p)
+                    if p != "0":
+                        my_plants.pop(my_plants.index(p))
                 if plant in my_plants:
                     users = state["users"]
                     now_pos[pos - 1] = plant
@@ -1659,7 +1725,7 @@ async def _(bot: Bot, event: GroupMessageEvent, state: T_State, team: str = ArgS
 async def _():
     if not os.path.exists(PVZ_OUTPUT_PATH):
         os.makedirs(PVZ_OUTPUT_PATH)
-    res = "欢迎了解植物大战僵尸v1.1.9\n\n" \
+    res = "欢迎了解植物大战僵尸v1.2.1\n\n" \
           "**************************************************\n\n" \
           "您可以通过使用关键字'查看背包'来查看您的背包\n\n" \
           "**************************************************\n" \
@@ -1679,15 +1745,62 @@ async def _():
           "**************************************************\n\n" \
           "通过'入侵'来操纵你的僵尸摧毁他人的防御\n例如'入侵 @龙城孤笑'\n\n" \
           "**************************************************\n\n" \
+          "管理员在使用前请务必仔细看文档，在更新插件之前请下载数据，以免数据丢失\n\n"\
+          "**************************************************\n\n" \
           "总之，在植物大战僵尸的世界中，祝你玩得开心，享受这个过程！\n\n\n" \
-          "插件中所有数据以及图片来源于 ’植物大战僵尸吧‘ 提供的全图鉴中v3.6.0，在此由衷感谢数据支持。\n\n\n" \
+          "插件中所有数据以及图片来源于 ’植物大战僵尸吧‘ 提供的全图鉴中v3.6.0，\n在此由衷感谢数据支持。\n\n\n" \
           "Create by longchengguxiao"
     await asyncio.sleep(1)
     draw_test(res, (255, 255, 153), Path(
         PVZ_OUTPUT_PATH, "help.png"), Path(FONT_PATH))
     await _help.finish(
-        MessageSegment.image("file:///" / PVZ_OUTPUT_PATH / "help.png") + MessageSegment.text("更详细的命令可以参考https://github.com/longchengguxiao/nonebot_plugin_pvz#%E5%91%BD%E4%BB%A4%E8%AF%A6%E8%A7%A3"), at_sender=True)
+        MessageSegment.image("file:///" / PVZ_OUTPUT_PATH / "help.png") + MessageSegment.text("建议使用前阅读更详细的命令https://github.com/longchengguxiao/nonebot_plugin_pvz#%E5%91%BD%E4%BB%A4%E8%AF%A6%E8%A7%A3"), at_sender=True)
 
+# 数据的上传与下载--------------------------------------------------------------------
+
+
+@download_data.handle()
+async def _():
+    await asyncio.sleep(1)
+    await download_data.send("收到请求")
+    await asyncio.sleep(1)
+
+
+@download_data.got("for_sure",
+                   prompt="本功能常用于更新包前的数据下载，请再次确认下载数据吗，这将会覆盖当前本地储存数据，回复'算了'或'取消'来取消操作")
+async def _(for_sure: str = ArgStr("for_sure")):
+    if for_sure in ["算了", "取消"]:
+        await download_data.finish("取消下载数据")
+    target_path = os.path.expanduser(
+        os.path.join('~', '.nonebot_plugin_pvz')
+    )
+    if not os.path.exists(target_path):
+        os.makedirs(target_path)
+    shutil.copyfile(Path(bag_path), os.path.join(target_path, 'bag.txt'))
+    shutil.copyfile(Path(lawn_path), os.path.join(target_path, 'lawn.txt'))
+    await download_data.finish("pvz用户数据已经成功保存")
+
+
+@upload_data.handle()
+async def _():
+    await asyncio.sleep(1)
+    await download_data.send("收到请求")
+    await asyncio.sleep(1)
+
+
+@upload_data.got("for_sure",
+                 prompt="本功能常用于更新包后的数据载入，请再次确认上传数据吗，这将会覆盖当前安装包内数据，回复'算了'或'取消'来取消操作")
+async def _(for_sure: str = ArgStr("for_sure")):
+    if for_sure in ["算了", "取消"]:
+        await download_data.finish("取消下载数据")
+    src_path = os.path.expanduser(
+        os.path.join('~', '.nonebot_plugin_pvz')
+    )
+    if not os.path.exists(src_path):
+        await upload_data.finish("未发现本地缓存的pvz用户数据，请先使用命令'pvz下载数据'后上传")
+    shutil.copyfile(os.path.join(src_path, 'bag.txt'), Path(bag_path))
+    shutil.copyfile(os.path.join(src_path, 'lawn.txt'), Path(lawn_path))
+    await download_data.finish("pvz用户数据已经成功保存")
 
 # 定时操作，更改签到数据----------------------------------------------------------------
 
